var init = (function (Clutter, Gio, GLib, GObject, St, PanelMenu, PopupMenu) {
    'use strict';

    const ExtensionUtils = imports.misc.extensionUtils;
    const Main = imports.ui.main;
    const ByteArray = imports.byteArray;
    const Me = ExtensionUtils.getCurrentExtension();
    // const _ = ExtensionUtils.gettext
    const _ = (text) => text;
    const debug = (str) => console.log(["[tpt] => ", str].join(''));
    const iconFrom = (filename) => {
        const p = [Me.path, 'icons', filename]
            .join('/')
            .replace(/\/+/igm, '/');
        return Gio.icon_new_for_string(p);
    };
    const UNIT = {
        celsius: "\u00b0C",
        fahrenheit: "\u00b0F",
        rpm: "RPM"
    };
    const ICON = {
        fan: {
            gicon: iconFrom('fan-symbolic.svg'),
            size: 12
        },
        cpu: {
            gicon: iconFrom('cpu-symbolic.svg'),
            size: 12
        },
        gpu: {
            gicon: iconFrom('gpu-symbolic.svg'),
            size: 14
        },
        hdd: {
            gicon: iconFrom('hdd-symbolic.svg'),
            size: 14
        },
    };
    class ConsoleUtil {
        constructor() {
            this._available = false;
            this._command = '';
            this._updated = false;
        }
        execute(callback) {
            try {
                if (this.available) {
                    const [ok, out, err] = GLib.spawn_command_line_sync(this.command);
                    if (!ok) {
                        throw new Error(ByteArray.toString(err));
                    }
                    if (ok && out) {
                        const str = ByteArray.toString(out);
                        return callback(str);
                    }
                }
            }
            catch (e) {
                // error here
                debug('catch err :: ' + e);
            }
        }
        set command(cmd) {
            this._command = cmd;
        }
        get command() {
            return this._command;
        }
        set available(flag) {
            this._available = flag;
        }
        get available() {
            return this._available;
        }
    }
    class IbmAcpiUtil extends ConsoleUtil {
        constructor() {
            super();
            this._data = {
                cpu: 0,
                gpu: 0,
                status: '...',
                speed: 0,
                level: '...'
            };
            const src = [
                '/proc/acpi/ibm/thermal',
                '/proc/acpi/ibm/fan'
            ];
            this.available =
                GLib.file_test(src[0], GLib.FileTest.EXISTS) &&
                    GLib.file_test(src[1], GLib.FileTest.EXISTS);
            if (this.available) {
                this.command = ['cat', src[0], src[1]].join(' ');
                // immediately get initial values
                this.update();
            }
        }
        // temperatures: 43 50 0 0 0 0 0 0
        // status: enabled
        // speed: 2314
        // level: auto
        // commands: level<level>(<level>is 0 - 7, auto, disengaged, full - speed)
        // commands: enable, disable
        // commands: watchdog<timeout>(<timeout>is 0(off), 1 - 120(seconds))
        parse(str) {
            const getVal = (str) => str
                .replace(/\t+/, " ")
                .split(': ')[1];
            const row = str.split("\n");
            const [cpu, gpu] = getVal(row[0]).split(' ');
            this._data = {
                cpu: parseInt(cpu),
                gpu: parseInt(gpu),
                status: getVal(row[1]),
                speed: parseInt(getVal(row[2])),
                level: getVal(row[3])
            };
        }
        update() {
            super.execute(this.parse.bind(this));
        }
        get cpu() {
            return this._data.cpu.toString();
        }
        get gpu() {
            return this._data.gpu.toString();
        }
        get status() {
            return this._data.status;
        }
        get speed() {
            return this._data.speed.toString();
        }
        get level() {
            return this._data.level;
        }
    }
    const ThermalTitle = GObject.registerClass(class ThermalTitle extends PopupMenu.PopupBaseMenuItem {
        constructor(title) {
            super({
                style_class: 'tpt-popup-title', reactive: false
            });
            this.setOrnament(PopupMenu.Ornament.HIDDEN);
            this._title = new St.Label({
                text: title,
                x_align: Clutter.ActorAlign.CENTER,
                x_expand: true
            });
            this.add_child(this._title);
        }
    });
    const ThermalItem = GObject.registerClass(class ThermalItem extends PopupMenu.PopupBaseMenuItem {
        constructor(data) {
            super({
                style_class: 'tpt-popup-item', reactive: false
            });
            this._id = data.id;
            if (data.icon || data.hideOrnament) {
                this.setOrnament(PopupMenu.Ornament.HIDDEN);
            }
            if (data.icon) {
                this.add_child(new St.Icon({
                    style_class: 'popup-menu-ornament',
                    icon_size: 14,
                    gicon: data.icon?.gicon,
                }));
            }
            this._label = new St.Label({
                style_class: 'label',
                text: data.label,
                x_align: Clutter.ActorAlign.START,
                x_expand: true
            });
            this.add_child(this._label);
            this._value = new St.Label({
                style_class: 'value',
                text: data.value
            });
            this.add_child(this._value);
            if (data?.unit) {
                this._unit = new St.Label({
                    style_class: 'unit',
                    text: data.unit
                });
                this.add_child(this._unit);
            }
        }
        update(value, unit, label) {
            this._value.set_text(value);
            if (unit)
                this._unit.set_text(unit);
            if (label)
                this._label.set_text(label);
        }
        get id() {
            return this._id;
        }
    });
    const IndicatorItem = GObject.registerClass(class IndicatorItem extends St.BoxLayout {
        constructor(id, icon, value, unit) {
            super({ style_class: 'tpt-item' });
            this._id = id;
            this._icon = new St.Icon({
                gicon: icon.gicon,
                icon_size: icon.size,
                style_class: 'tpt-item-icon',
                y_align: Clutter.ActorAlign.CENTER
            });
            this._value = new St.Label({
                text: value,
                style_class: 'tpt-item-value',
                y_align: Clutter.ActorAlign.CENTER
            });
            this._unit = new St.Label({
                text: unit,
                style_class: 'tpt-item-unit',
                y_align: Clutter.ActorAlign.CENTER
            });
            this.add_child(this._icon);
            this.add_child(this._value);
            this.add_child(this._unit);
        }
        update(value, unit) {
            this._value.set_text(value);
            if (unit)
                this._unit.set_text(unit);
        }
        get id() {
            return this._id;
        }
    });
    const Indicator = GObject.registerClass(class Indicator extends PanelMenu.Button {
        constructor(...args) {
            super(...args);
            this._elements = ['cpu', 'gpu', 'speed'];
            this._indicators = [];
            this._popup = [];
            // utility fn to convert celsius to fahrenheit
            this._toFahrenheit = (temp) => (temp * (9 / 5)) + 32;
            // perform update
            this._update = () => {
                if (this._tpAcpi._available) {
                    this._tpAcpi.update();
                    // update indicator values
                    for (let i = 0; i < this._indicators.length; i++) {
                        const item = this._indicators[i];
                        const value = this._tpAcpi[this._indicators[i].id];
                        item.update(value);
                    }
                    // update popup values
                    for (let i = 0; i < this._popup.length; i++) {
                        for (let j = 0; j < this._popup[i].items.length; j++) {
                            const item = this._popup[i].items[j];
                            const value = this._tpAcpi[item.id];
                            item.update(value);
                        }
                    }
                }
            };
            this._tpAcpi = new IbmAcpiUtil();
            this._layout = new St.BoxLayout({
                style_class: 'tpt-button'
            });
            if (this._tpAcpi.available) {
                // add cpu to indicators
                if (this.element('cpu')) {
                    this._indicators.push(new IndicatorItem('cpu', ICON.cpu, this._tpAcpi.cpu, UNIT.celsius));
                }
                // add gpu to indicators
                if (this.element('gpu')) {
                    this._indicators.push(new IndicatorItem('gpu', ICON.gpu, this._tpAcpi.gpu, UNIT.celsius));
                }
                // add speed to indicators
                if (this.element('speed')) {
                    this._indicators.push(new IndicatorItem('speed', ICON.fan, this._tpAcpi.speed, UNIT.rpm));
                }
                // place indicators on layout
                for (let i = 0; i < this._indicators.length; i++) {
                    this._layout.add_child(this._indicators[i]);
                }
                this.appendPopupMenu();
            }
            else {
                this._layout.add_child(new St.Label({ text: "N/A" }));
            }
            this.add_actor(this._layout);
            if (this._tpAcpi.available) {
                this._updateInterval = setInterval(this._update.bind(this), 5000);
                this.connect("destroy", this._destroy.bind(this));
            }
        }
        // cleanup on destroy
        _destroy() {
            if (this._updateInterval?.get_id()) {
                this._updateInterval.destroy();
            }
        }
        // is element enabled?
        element(key) {
            return this._elements.includes(key);
        }
        appendPopupMenu() {
            // Sensors
            // this.menu.addMenuItem(new ThermalTitle("Sensors"))
            // this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem())
            // tpAcpi groups
            if (this._tpAcpi.available) {
                // add ACPI group to popup
                this._popup.push({
                    title: "ACPI",
                    config: [
                        { id: 'cpu', label: 'CPU', value: this._tpAcpi.cpu, unit: UNIT.celsius, icon: ICON.cpu },
                        { id: 'gpu', label: 'GPU', value: this._tpAcpi.gpu, unit: UNIT.celsius, icon: ICON.gpu }
                    ],
                    items: []
                });
                // add Fan control group to popup
                this._popup.push({
                    title: "Fan control",
                    config: [
                        { id: 'status', label: 'Status', value: this._tpAcpi.status, hideOrnament: true },
                        { id: 'speed', label: 'Speed', value: this._tpAcpi.speed, unit: UNIT.rpm, hideOrnament: true },
                        { id: 'level', label: 'Level', value: this._tpAcpi.level, hideOrnament: true }
                    ],
                    items: []
                });
                // place popup items on layout
                for (let i = 0; i < this._popup.length; i++) {
                    const { config, title } = this._popup[i];
                    this.menu.addMenuItem(new ThermalTitle(title));
                    for (let j = 0; j < config.length; j++) {
                        const item = new ThermalItem(config[j]);
                        this._popup[i].items.push(item);
                        this.menu.addMenuItem(item);
                    }
                    if (i < (this._popup.length - 1)) {
                        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem);
                    }
                }
            }
        }
    });
    class ThinkPadThermal {
        constructor(uuid) {
            this._uuid = uuid;
        }
        enable() {
            this._indicator = new Indicator(0.0, _('ThinkPad Thermal'));
            Main.panel.addToStatusArea(this._uuid, this._indicator);
        }
        disable() {
            this._indicator.destroy();
            this._indicator = null;
        }
    }
    function extension (meta) {
        return new ThinkPadThermal(meta.uuid);
    }

    return extension;

})(imports.gi.Clutter, imports.gi.Gio, imports.gi.GLib, imports.gi.GObject, imports.gi.St, imports.ui.panelMenu, imports.ui.popupMenu);
